var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

import React from "react";
import graphql from "graphql-anywhere";
import glamorous from "glamorous";
import { Broadcast, Subscriber } from "react-broadcast";

var smoosh = function smoosh(object) {
    return Object.assign.apply(Object, [{}].concat(function _flatten(objectBit) {
        var _ref;

        var path = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : "";

        return (_ref = []).concat.apply(_ref, Object.keys(objectBit).map(function (key) {
            var _ref2;

            return _typeof(objectBit[key]) === "object" ? _flatten(objectBit[key]) : (_ref2 = {}, _ref2[key] = objectBit[key], _ref2);
        }));
    }(object)));
};

var resolver = function resolver(fieldName) {
    var root = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    var args = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
    var context = arguments[3];
    var _ref3 = arguments[4];
    var resultKey = _ref3.resultKey;

    // if it's an aliased query add alias as prop
    if (fieldName !== resultKey) {
        return _extends({}, root[fieldName], args, {
            prop: resultKey
        });
    }

    var res = root[fieldName];
    var rootUnit = root && root.unit;
    var argsUnit = args && args.unit;

    if (argsUnit || rootUnit) {
        var unit = argsUnit || rootUnit;
        res = root[fieldName] + unit;
    }

    // if has prop then use it as the key
    if (root.prop) {
        var _ref4;

        return _ref4 = {}, _ref4[root.prop] = res, _ref4;
    }

    return res;
};

// Main function to generate glamorous component with styles
var gqlCSS = function gqlCSS(styles) {
    return function (query) {
        var component = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : "div";

        var generatedStyles = graphql(resolver, query, styles, null, null);
        var smooshedStyles = smoosh(generatedStyles);

        // Returns just smooshed styles if component is false
        if (component === false) {
            return smooshedStyles;
        }

        // Create glamorous component
        return glamorous(component)(smooshedStyles);
    };
};

// Also export component for more declarative API
var GqlCSS = function GqlCSS(_ref5) {
    var styles = _ref5.styles,
        query = _ref5.query,
        _ref5$component = _ref5.component,
        component = _ref5$component === undefined ? "div" : _ref5$component,
        rest = _objectWithoutProperties(_ref5, ["styles", "query", "component"]);

    return React.createElement(
        Subscriber,
        { quiet: true, channel: "graphqlcss" },
        function (contextStyles) {
            var Component = gqlCSS(styles || contextStyles || "")(query, component);
            return React.createElement(Component, rest);
        }
    );
};

// Export provider to broadcast styles to child GqlCSS components
export { GqlCSS };
export var GqlCSSProvider = function GqlCSSProvider(_ref6) {
    var styles = _ref6.styles,
        children = _ref6.children;
    return React.createElement(
        Broadcast,
        { channel: "graphqlcss", value: styles },
        React.createElement(
            "span",
            null,
            children
        )
    );
};

// Export HOC - uses render props underneath, because it's awesome
export var withGqlCSS = function withGqlCSS(styles, query) {
    return function (Component) {
        return function (props) {
            return React.createElement(
                WithGqlCSS,
                { styles: styles, query: query },
                function (_ref7) {
                    var gqlStyles = _ref7.gqlStyles;
                    return React.createElement(Component, _extends({}, props, { gqlStyles: gqlStyles }));
                }
            );
        };
    };
};

// Export render props component
var WithGqlCSS = function WithGqlCSS(_ref8) {
    var styles = _ref8.styles,
        query = _ref8.query,
        children = _ref8.children,
        rest = _objectWithoutProperties(_ref8, ["styles", "query", "children"]);

    var processedStyles = gqlCSS(styles)(query, false);
    return children(_extends({ gqlStyles: processedStyles }, rest));
};

export { WithGqlCSS };
export default gqlCSS;